@@ -311,7 +311,7 @@ int opkg_install_package(const char *package_url,
     progress(&pdata, 75, progress_callback, user_data);
 
     /* unpack the package */
-    err = opkg_install_pkg(new);
+    err = opkg_install_pkg(new, NULL);
 
     if (err) {
         return -1;
