@@ -4,6 +4,8 @@
 
    Copyright (C) 2015 National Instruments Corp.
 
+   SPDX-License-Identifier: GPL-2.0-or-later
+
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2, or (at
@@ -334,8 +336,9 @@ static int constraint_to_solv_flags(version_constraint_t constraint)
 }
 
 /* This transforms a compound depend [e.g. A (=3.0)|C (>=2.0) ]
-   into an id usable by libsolv. */
-static Id dep2id(Pool *pool, compound_depend_t *dep)
+   into an id usable by libsolv.  Use skip_installed to skip
+   dependencies that are already installed */
+static Id dep2id(Pool *pool, compound_depend_t *dep, int skip_installed)
 {
     Id nameId = 0;
     Id versionId = 0;
@@ -348,6 +351,10 @@ static Id dep2id(Pool *pool, compound_depend_t *dep)
     for (i = 0; i < dep->possibility_count; ++i) {
         depend_t *depend = dep->possibilities[i];
 
+        if (skip_installed && !pkg_hash_fetch_installed_by_name(depend->pkg->name)) {
+            continue;
+        }
+
         nameId = pool_str2id(pool, depend->pkg->name, 1);
         if (depend->version) {
             versionId = pool_str2id(pool, depend->version, 1);
@@ -365,7 +372,7 @@ static Id dep2id(Pool *pool, compound_depend_t *dep)
     return dependId;
 }
 
-static void pkg2solvable(pkg_t *pkg, Solvable *solvable_out)
+static void pkg2solvable(pkg_t *pkg, Solvable *solvable_out, int installed)
 {
     if (!solvable_out) {
         opkg_msg(ERROR, "Solvable undefined!\n");
@@ -398,7 +405,7 @@ static void pkg2solvable(pkg_t *pkg, Solvable *solvable_out)
 
         for (i = 0; i < deps_count; ++i) {
             compound_depend_t dep = pkg->depends[i];
-            dependId = dep2id(pool, &dep);
+            dependId = dep2id(pool, &dep, 0);
             switch (dep.type) {
             /* pre-depends is not supported by opkg. However, the
                field is parsed in pkg_parse. This will handle the
@@ -419,8 +426,10 @@ static void pkg2solvable(pkg_t *pkg, Solvable *solvable_out)
             case RECOMMEND:
                 opkg_msg(DEBUG2, "%s recommends %s\n", pkg->name,
                          pool_dep2str(pool, dependId));
+                // If the package is installed, skip not installed recommends dependencies
+                // This is needed to avoid installing missing recommends of installed packages
                 solvable_out->recommends = repo_addid_dep(repo,
-                         solvable_out->recommends, dependId, 0);
+                         solvable_out->recommends, dep2id(pool, &dep, installed), 0);
                 break;
             case SUGGEST:
                 opkg_msg(DEBUG2, "%s suggests %s\n", pkg->name,
@@ -439,7 +448,7 @@ static void pkg2solvable(pkg_t *pkg, Solvable *solvable_out)
         for (i = 0; i < pkg->conflicts_count; i++) {
             compound_depend_t dep = pkg->conflicts[i];
 
-            dependId = dep2id(pool, &dep);
+            dependId = dep2id(pool, &dep, 0);
             opkg_msg(DEBUG2, "%s conflicts %s\n", pkg->name,
                      pool_dep2str(pool, dependId));
             solvable_out->conflicts = repo_addid_dep(repo,
@@ -472,7 +481,7 @@ static void pkg2solvable(pkg_t *pkg, Solvable *solvable_out)
         for (i = 0; i < pkg->replaces_count; i++) {
             compound_depend_t dep = pkg->replaces[i];
 
-            Id replacesId = dep2id(pool, &dep);
+            Id replacesId = dep2id(pool, &dep, 0);
             opkg_msg(DEBUG2, "%s replaces %s\n", pkg->name,
                      pool_dep2str(pool, replacesId));
             solvable_out->obsoletes = repo_addid_dep(repo,
@@ -506,10 +515,10 @@ static void populate_installed_repo(libsolv_solver_t *libsolv_solver)
                                               solvable_id);
 
         /* set solvable attributes */
-        pkg2solvable(pkg, solvable);
+        pkg2solvable(pkg, solvable, 1);
 
         /* if the package is in ignore-recommends-list, disfavor installation */
-        if (str_list_contains(&opkg_config->ignore_recommends_list, pkg->name)) {
+        if (str_list_contains(&opkg_config->ignore_recommends_list, pkg->name, 1)) {
             opkg_message(NOTICE, "Disfavor package: %s\n",
                          pkg->name);
             what = pool_str2id(libsolv_solver->pool, pkg->name, 1);
@@ -559,7 +568,7 @@ static void populate_available_repos(libsolv_solver_t *libsolv_solver)
         pkg_t *pkg = available_pkgs->pkgs[i];
 
         /* if the package is marked as excluded, skip it */
-        if (str_list_contains(&opkg_config->exclude_list, pkg->name))
+        if (str_list_contains(&opkg_config->exclude_list, pkg->name, 1))
             continue;
 
         /* if the package is installed or unpacked, skip it */
@@ -616,10 +625,10 @@ static void populate_available_repos(libsolv_solver_t *libsolv_solver)
 
         /* set solvable attributes using the package */
         solvable = pool_id2solvable(libsolv_solver->pool, solvable_id);
-        pkg2solvable(pkg, solvable);
+        pkg2solvable(pkg, solvable, 0);
 
         /* if the package is in ignore-recommends-list, disfavor installation */
-        if (str_list_contains(&opkg_config->ignore_recommends_list, pkg->name)) {
+        if (str_list_contains(&opkg_config->ignore_recommends_list, pkg->name, 1)) {
             opkg_message(NOTICE, "Disfavor package: %s\n",
                          pkg->name);
             what = pool_str2id(libsolv_solver->pool, pkg->name, 1);
@@ -797,6 +806,24 @@ static void libsolv_solver_add_job(libsolv_solver_t *libsolv_solver,
     }
 }
 
+static void libsolv_print_all_problem_rules(libsolv_solver_t *libsolv_solver, Id problem)
+{
+    Queue rules;
+    int i;
+
+    queue_init(&rules);
+    solver_findallproblemrules(libsolv_solver->solver, problem, &rules);
+    for (i = 0; i < rules.count; i++) {
+        Id type, source, target, dep;
+        Id r = rules.elements[i];
+        type = solver_ruleinfo(libsolv_solver->solver, r, &source, &target, &dep);
+        opkg_message(ERROR, "  - %s\n",
+                     solver_problemruleinfo2str(libsolv_solver->solver,
+                                                type, source, target, dep));
+    }
+    queue_free(&rules);
+}
+
 static int libsolv_solver_solve(libsolv_solver_t *libsolv_solver)
 {
     int problem_count = solver_solve(libsolv_solver->solver,
@@ -810,8 +837,7 @@ static int libsolv_solver_solve(libsolv_solver_t *libsolv_solver)
         /* problems start at 1, not 0 */
         for (problem = 1; problem <= problem_count; problem++) {
             opkg_message(ERROR, "Problem %d/%d:\n", problem, problem_count);
-            opkg_message(ERROR, "  - %s\n",
-                         solver_problem2str(libsolv_solver->solver, problem));
+            libsolv_print_all_problem_rules(libsolv_solver, problem);
             opkg_message(ERROR, "\n");
 
             int solution_count = solver_solution_count(libsolv_solver->solver,
@@ -860,7 +886,8 @@ static int libsolv_solver_transaction_preamble(libsolv_solver_t *libsolv_solver,
         Id typeId = transaction_type(transaction, stepId,
                 SOLVER_TRANSACTION_SHOW_ACTIVE |
                 SOLVER_TRANSACTION_CHANGE_IS_REINSTALL |
-                SOLVER_TRANSACTION_SHOW_OBSOLETES);
+                SOLVER_TRANSACTION_SHOW_OBSOLETES |
+                SOLVER_TRANSACTION_OBSOLETE_IS_UPGRADE);
 
         const char *pkg_name = pool_id2str(libsolv_solver->pool, solvable->name);
         const char *evr = pool_id2str(libsolv_solver->pool, solvable->evr);
@@ -907,10 +934,11 @@ static int libsolv_solver_execute_transaction(libsolv_solver_t *libsolv_solver)
             Id typeId = transaction_type(transaction, stepId,
                     SOLVER_TRANSACTION_SHOW_ACTIVE |
                     SOLVER_TRANSACTION_CHANGE_IS_REINSTALL |
-                    SOLVER_TRANSACTION_SHOW_OBSOLETES);
+                    SOLVER_TRANSACTION_SHOW_OBSOLETES |
+                    SOLVER_TRANSACTION_OBSOLETE_IS_UPGRADE);
 
             pkg = pkgs->pkgs[i];
-            pkg_t *old, *obs = NULL;
+            pkg_t *old = NULL, *obs = NULL;
 
             Id decision_rule;
 
@@ -922,15 +950,6 @@ static int libsolv_solver_execute_transaction(libsolv_solver_t *libsolv_solver)
                     goto CLEANUP;
                 }
                 break;
-            case SOLVER_TRANSACTION_OBSOLETES:
-                /* Replaces operations are expressed in two steps: the first one is a SOLVER_TRANSACTION_OBSOLETES, with the name of
-                 * the replacer package. The second one is a SOLVER_TRANSACTION_IGNORE, with the name of the replacee */
-                obs = pkgs->pkgs[i + 1];
-                ret = opkg_remove_pkg(obs);
-                if (ret) {
-                    err = -1;
-                    goto CLEANUP;
-                }
             case SOLVER_TRANSACTION_DOWNGRADE:
             case SOLVER_TRANSACTION_REINSTALL:
             case SOLVER_TRANSACTION_INSTALL:
@@ -951,14 +970,34 @@ static int libsolv_solver_execute_transaction(libsolv_solver_t *libsolv_solver)
                     opkg_message(NOTICE, "Installing %s (%s) on %s\n",
                                  pkg->name, pkg->version, pkg->dest->name);
                 }
-                ret = opkg_install_pkg(pkg);
+                ret = opkg_install_pkg(pkg, NULL);
                 if (ret) {
                     err = -1;
                     goto CLEANUP;
                 }
                 break;
             case SOLVER_TRANSACTION_UPGRADE:
-                old = pkg_hash_fetch_installed_by_name(pkg->name);
+                  /* An upgrade due to a package obsoleting another one (SOLVER_TRANSACTION_OBSOLETE_IS_UPGRADE)
+                   * will show as an upgrade operation (new pkg) AND a ignore operation (old package)   */
+                  if ((i+1) < transaction->steps.count) {
+                    Id nextStepId = transaction->steps.elements[i+1];
+                    Id nextTypeId = transaction_type(transaction, nextStepId,
+                        SOLVER_TRANSACTION_SHOW_ACTIVE |
+                        SOLVER_TRANSACTION_CHANGE_IS_REINSTALL |
+                        SOLVER_TRANSACTION_SHOW_OBSOLETES |
+                        SOLVER_TRANSACTION_OBSOLETE_IS_UPGRADE);
+
+                   if (nextTypeId == SOLVER_TRANSACTION_IGNORE) {
+                        obs = pkgs->pkgs[i+1];
+                        old = pkg_hash_fetch_installed_by_name(obs->name);
+                    }
+                }
+
+                /* If this is not an OBSOLETE operation, check to see if there is an older pkg installed */
+                if (!obs) {
+                    old = pkg_hash_fetch_installed_by_name(pkg->name);
+                }
+
                 /* if an old version was found set the new package's
                    autoinstalled status to that of the old package. */
                 if (old) {
@@ -970,8 +1009,8 @@ static int libsolv_solver_execute_transaction(libsolv_solver_t *libsolv_solver)
                     if (!opkg_config->download_only) {
                         char *old_version = pkg_version_str_alloc(old);
 
-                        opkg_message(NOTICE, "Upgrading %s from %s to %s on %s\n",
-                                     pkg->name, old_version, pkg->version, pkg->dest->name);
+                        opkg_message(NOTICE, "Upgrading %s (%s) to %s (%s) on %s\n",
+                                     old->name, old_version, pkg->name, pkg->version, pkg->dest->name);
                         free(old_version);
                     }
                 } else {
@@ -983,7 +1022,7 @@ static int libsolv_solver_execute_transaction(libsolv_solver_t *libsolv_solver)
                     }
                 }
 
-                ret = opkg_install_pkg(pkg);
+                ret = opkg_install_pkg(pkg, old);
                 if (ret) {
                     err = -1;
                     goto CLEANUP;
