@@ -99,7 +99,7 @@ class Opk:
 				tar.add(control_file)
 				tar.add(data_file)
 		else:
-			os.system("ar q {0} {1} {2} {3}\
+			os.system("ar q -U {0} {1} {2} {3}\
 					2>/dev/null".format(filename,
 						'debian-binary',
 						control_file, data_file))
